=== modified file 'NEWS'
--- NEWS	2005-11-11 00:10:50 +0000
+++ NEWS	2005-11-13 20:14:22 +0000
@@ -24,6 +24,9 @@
     * Fix bug in reporting http errors that don't have an http error code.
       (Martin Pool)
 
+    * Stores with some compressed texts and some uncompressed texts are now
+      able to be used. (John A Meinel)
+
   TESTING:
 
     * Fix selftest asking for passwords when there are no SFTP keys.  

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-11-09 17:24:18 +0000
+++ bzrlib/branch.py	2005-11-13 20:14:22 +0000
@@ -44,7 +44,6 @@
 from bzrlib.tree import EmptyTree, RevisionTree
 from bzrlib.inventory import Inventory
 from bzrlib.store import copy_all
-from bzrlib.store.compressed_text import CompressedTextStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.weave import WeaveStore
 from bzrlib.testament import Testament
@@ -237,12 +236,9 @@
             # some existing branches where there's a mixture; we probably 
             # still want the option to look for both.
             relpath = self._rel_controlfilename(name)
-            if compressed:
-                store = CompressedTextStore(self._transport.clone(relpath),
-                                            prefixed=prefixed)
-            else:
-                store = TextStore(self._transport.clone(relpath),
-                                  prefixed=prefixed)
+            store = TextStore(self._transport.clone(relpath),
+                              prefixed=prefixed,
+                              compressed=compressed)
             #if self._transport.should_cache():
             #    cache_path = os.path.join(self.cache_root, name)
             #    os.mkdir(cache_path)

=== modified file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	2005-10-31 20:52:50 +0000
+++ bzrlib/selftest/teststore.py	2005-11-10 02:11:13 +0000
@@ -18,12 +18,12 @@
 
 from cStringIO import StringIO
 import os
+import gzip
 
 from bzrlib.errors import BzrError, UnlistableStore
 from bzrlib.store import copy_all
 from bzrlib.transport.local import LocalTransport
 from bzrlib.transport import NoSuchFile
-from bzrlib.store.compressed_text import CompressedTextStore
 from bzrlib.store.text import TextStore
 from bzrlib.selftest import TestCase, TestCaseInTempDir
 import bzrlib.store as store
@@ -82,7 +82,7 @@
 
     def get_store(self, path='.'):
         t = LocalTransport(path)
-        return CompressedTextStore(t)
+        return TextStore(t, compressed=True)
 
     def test_total_size(self):
         store = self.get_store('.')
@@ -93,9 +93,9 @@
         self.assertEqual(store.total_size(), (2, 55))
         
     def test__relpath_suffixed(self):
-        my_store = CompressedTextStore(MockTransport(), True)
+        my_store = TextStore(MockTransport(), True, compressed=True)
         my_store.register_suffix('dsc')
-        self.assertEqual('45/foo.dsc.gz', my_store._relpath('foo', ['dsc']))
+        self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
 
 
 class TestMemoryStore(TestCase):
@@ -141,7 +141,7 @@
 
     def get_store(self, path='.'):
         t = LocalTransport(path)
-        return TextStore(t)
+        return TextStore(t, compressed=False)
 
     def test_total_size(self):
         store = self.get_store()
@@ -155,6 +155,41 @@
         # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
 
 
+class TestMixedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path='.', compressed=True):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=compressed)
+
+    def test_get_mixed(self):
+        cs = self.get_store('.', compressed=True)
+        s = self.get_store('.', compressed=False)
+        cs.add(StringIO('hello there'), 'a')
+
+        self.failUnlessExists('a.gz')
+        self.failIf(os.path.lexists('a'))
+
+        self.assertEquals(gzip.GzipFile('a.gz').read(), 'hello there')
+
+        self.assertEquals(cs.has_id('a'), True)
+        self.assertEquals(s.has_id('a'), True)
+        self.assertEquals(cs.get('a').read(), 'hello there')
+        self.assertEquals(s.get('a').read(), 'hello there')
+        
+        self.assertRaises(BzrError, s.add, StringIO('goodbye'), 'a')
+
+        s.add(StringIO('goodbye'), 'b')
+        self.failUnlessExists('b')
+        self.failIf(os.path.lexists('b.gz'))
+        self.assertEquals(open('b').read(), 'goodbye')
+
+        self.assertEquals(cs.has_id('b'), True)
+        self.assertEquals(s.has_id('b'), True)
+        self.assertEquals(cs.get('b').read(), 'goodbye')
+        self.assertEquals(s.get('b').read(), 'goodbye')
+        
+        self.assertRaises(BzrError, cs.add, StringIO('again'), 'b')
+
 class MockTransport(transport.Transport):
     """A fake transport for testing with."""
 
@@ -232,18 +267,18 @@
 
     def test__relpath_simple_suffixed(self):
         my_store = store.TransportStore(MockTransport())
-        my_store.register_suffix('gz')
         my_store.register_suffix('bar')
-        self.assertEqual('foo.gz', my_store._relpath('foo', ['gz']))
-        self.assertEqual('foo.gz.bar', my_store._relpath('foo', ['gz', 'bar']))
+        my_store.register_suffix('baz')
+        self.assertEqual('foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('foo.bar.baz', my_store._relpath('foo', ['bar', 'baz']))
 
     def test__relpath_prefixed_suffixed(self):
         my_store = store.TransportStore(MockTransport(), True)
-        my_store.register_suffix('gz')
         my_store.register_suffix('bar')
-        self.assertEqual('45/foo.gz', my_store._relpath('foo', ['gz']))
-        self.assertEqual('45/foo.gz.bar',
-                         my_store._relpath('foo', ['gz', 'bar']))
+        my_store.register_suffix('baz')
+        self.assertEqual('45/foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('45/foo.bar.baz',
+                         my_store._relpath('foo', ['bar', 'baz']))
 
     def test_add_simple(self):
         stream = StringIO("content")
@@ -271,8 +306,10 @@
         my_store.add(stream, "foo", 'dsc')
         self.assertEqual([("_add", "45/foo.dsc", stream)], my_store._calls)
 
-    def get_populated_store(self, prefixed=False, store_class=TextStore):
-        my_store = store_class(MemoryTransport(), prefixed)
+    def get_populated_store(self, prefixed=False,
+            store_class=TextStore, compressed=False):
+        my_store = store_class(MemoryTransport(), prefixed,
+                               compressed=compressed)
         my_store.register_suffix('sig')
         stream = StringIO("signature")
         my_store.add(stream, "foo", 'sig')
@@ -321,7 +358,7 @@
                          my_store.get('missing', 'sig').read())
 
     def test___iter__no_suffix(self):
-        my_store = TextStore(MemoryTransport(), False)
+        my_store = TextStore(MemoryTransport(), False, compressed=False)
         stream = StringIO("content")
         my_store.add(stream, "foo")
         self.assertEqual(set(['foo']),
@@ -336,17 +373,17 @@
     def test___iter__compressed(self):
         self.assertEqual(set(['foo']),
                          set(self.get_populated_store(
-                             store_class=CompressedTextStore).__iter__()))
+                             compressed=True).__iter__()))
         self.assertEqual(set(['foo']),
                          set(self.get_populated_store(
-                             True, CompressedTextStore).__iter__()))
+                             True, compressed=True).__iter__()))
 
     def test___len__(self):
         self.assertEqual(1, len(self.get_populated_store()))
 
     def test_copy_suffixes(self):
         from_store = self.get_populated_store()
-        to_store = CompressedTextStore(MemoryTransport(), True)
+        to_store = TextStore(MemoryTransport(), True, compressed=True)
         to_store.register_suffix('sig')
         copy_all(from_store, to_store)
         self.assertEqual(1, len(to_store))

=== modified file 'bzrlib/selftest/testtransport.py'
--- bzrlib/selftest/testtransport.py	2005-10-19 07:25:40 +0000
+++ bzrlib/selftest/testtransport.py	2005-11-10 01:31:41 +0000
@@ -65,8 +65,12 @@
         self.assertEqual(t.has(urlescape('%')), True)
         self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
                 [True, True, False, False, True, False, True, False])
+        self.assertEqual(t.has_any(['a', 'b', 'c']), True)
+        self.assertEqual(t.has_any(['c', 'd', 'f', urlescape('%%')]), False)
         self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
                 [True, True, False, False, True, False, True, False])
+        self.assertEqual(t.has_any(['c', 'c', 'c']), False)
+        self.assertEqual(t.has_any(['b', 'b', 'b']), True)
 
     def test_get(self):
         t = self.get_transport()
@@ -291,6 +295,15 @@
             self.assertEquals(open(f).read(),
                     open(os.path.join(dtmp_base, f)).read())
 
+        # Test that copying into a missing directory raises
+        # NoSuchFile
+        os.mkdir('e')
+        open('e/f', 'wb').write('contents of e')
+        self.assertRaises(NoSuchFile, t.copy_to, ['e/f'], local_t)
+
+        os.mkdir(os.path.join(dtmp_base, 'e'))
+        t.copy_to(['e/f'], local_t)
+
         del dtmp, dtmp_base, local_t
 
         dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-10-23 11:46:19 +0000
+++ bzrlib/store/__init__.py	2005-11-10 02:11:13 +0000
@@ -54,15 +54,15 @@
     def __len__(self):
         raise NotImplementedError('Children should define their length')
 
-    def get(self, file_id, suffix=None):
+    def get(self, fileid, suffix=None):
         """Returns a file reading from a particular entry.
         
-        If suffix is present, retrieve the named suffix for file_id.
+        If suffix is present, retrieve the named suffix for fileid.
         """
         raise NotImplementedError
 
     def __getitem__(self, fileid):
-        """DEPRECATED. Please use .get(file_id) instead."""
+        """DEPRECATED. Please use .get(fileid) instead."""
         raise NotImplementedError
 
     #def __contains__(self, fileid):
@@ -76,8 +76,8 @@
         """Add a file object f to the store accessible from the given fileid"""
         raise NotImplementedError('Children of Store must define their method of adding entries.')
 
-    def has_id(self, file_id, suffix=None):
-        """Return True or false for the presence of file_id in the store.
+    def has_id(self, fileid, suffix=None):
+        """Return True or false for the presence of fileid in the store.
         
         suffix, if present, is a per file suffix, i.e. for digital signature 
         data."""
@@ -136,6 +136,7 @@
         should call this if they have no optimised facility for a 
         specific 'other'.
         """
+        mutter('Store._copy_one: %r', fileid)
         f = other.get(fileid, suffix)
         self.add(f, fileid, suffix)
 
@@ -148,22 +149,24 @@
 
         f -- A file-like object, or string
         """
-        mutter("add store entry %r" % (fileid))
+        mutter("add store entry %r", fileid)
         
-        if suffix is not None:
-            fn = self._relpath(fileid, [suffix])
-        else:
-            fn = self._relpath(fileid)
-        if self._transport.has(fn):
-            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
-
-        if self._prefixed:
-            try:
-                self._transport.mkdir(hash_prefix(fileid)[:-1])
-            except errors.FileExists:
-                pass
-
-        self._add(fn, f)
+        names = self._id_to_names(fileid, suffix)
+        if self._transport.has_any(names):
+            raise BzrError("store %r already contains id %r" 
+                           % (self._transport.base, fileid))
+
+        # Most of the time, just adding the file will work
+        # if we find a time where it fails, (because the dir
+        # doesn't exist), then create the dir, and try again
+        self._add(names[0], f)
+
+
+    def _add(self, relpath, f):
+        """Actually add the file to the given location.
+        This should be overridden by children.
+        """
+        raise NotImplementedError('children need to implement this function.')
 
     def _check_fileid(self, fileid):
         if not isinstance(fileid, basestring):
@@ -171,13 +174,33 @@
         if '\\' in fileid or '/' in fileid:
             raise ValueError("invalid store id %r" % fileid)
 
-    def has_id(self, fileid, suffix=None):
-        """See Store.has_id."""
+    def _id_to_names(self, fileid, suffix):
+        """Return the names in the expected order"""
         if suffix is not None:
             fn = self._relpath(fileid, [suffix])
         else:
             fn = self._relpath(fileid)
-        return self._transport.has(fn)
+
+        fn_gz = fn + '.gz'
+        if self._compressed:
+            return fn_gz, fn
+        else:
+            return fn, fn_gz
+
+    def has_id(self, fileid, suffix=None):
+        """See Store.has_id."""
+        return self._transport.has_any(self._id_to_names(fileid, suffix))
+
+    def _get_name(self, fileid, suffix=None):
+        """A special check, which returns the name of an existing file.
+        
+        This is similar in spirit to 'has_id', but it is designed
+        to return information about which file the store has.
+        """
+        for name in self._id_to_names(fileid, suffix=suffix):
+            if self._transport.has(name):
+                return name
+        return None
 
     def _get(self, filename):
         """Return an vanilla file stream for clients to read from.
@@ -189,22 +212,20 @@
 
     def get(self, fileid, suffix=None):
         """See Store.get()."""
-        if suffix is None or suffix == 'gz':
-            fn = self._relpath(fileid)
-        else:
-            fn = self._relpath(fileid, [suffix])
-        try:
-            return self._get(fn)
-        except errors.NoSuchFile:
-            raise KeyError(fileid)
+        names = self._id_to_names(fileid, suffix)
+        for name in names:
+            try:
+                return self._get(name)
+            except errors.NoSuchFile:
+                pass
+        raise KeyError(fileid)
 
-    def __init__(self, a_transport, prefixed=False):
+    def __init__(self, a_transport, prefixed=False, compressed=False):
         assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
         self._transport = a_transport
         self._prefixed = prefixed
-        # conflating the .gz extension and user suffixes was a mistake.
-        # RBC 20051017 - TODO SOON, separate them again.
+        self._compressed = compressed
         self._suffixes = set()
 
     def _iter_files_recursive(self):
@@ -229,12 +250,15 @@
     def __len__(self):
         return len(list(self.__iter__()))
 
-    def _relpath(self, fileid, suffixes=[]):
+    def _relpath(self, fileid, suffixes=None):
         self._check_fileid(fileid)
-        for suffix in suffixes:
-            if not suffix in self._suffixes:
-                raise ValueError("Unregistered suffix %r" % suffix)
-            self._check_fileid(suffix)
+        if suffixes:
+            for suffix in suffixes:
+                if not suffix in self._suffixes:
+                    raise ValueError("Unregistered suffix %r" % suffix)
+                self._check_fileid(suffix)
+        else:
+            suffixes = []
         if self._prefixed:
             path = [hash_prefix(fileid) + fileid]
         else:
@@ -257,6 +281,8 @@
     def register_suffix(self, suffix):
         """Register a suffix as being expected in this store."""
         self._check_fileid(suffix)
+        if suffix == 'gz':
+            raise ValueError('You cannot register the "gz" suffix.')
         self._suffixes.add(suffix)
 
     def total_size(self):
@@ -293,7 +319,7 @@
         self.cache_store = store.__class__(LocalTransport(cache_dir))
 
     def get(self, id):
-        mutter("Cache add %s" % id)
+        mutter("Cache add %s", id)
         if id not in self.cache_store:
             self.cache_store.add(self.source_store.get(id), id)
         return self.cache_store.get(id)
@@ -303,7 +329,7 @@
         if self.cache_store.has_id(fileid, suffix):
             return True
         if self.source_store.has_id(fileid, suffix):
-            # We could copy at this time
+            # We could asynchronously copy at this time
             return True
         return False
 
@@ -314,8 +340,9 @@
     if not store_from.listable():
         raise UnlistableStore(store_from)
     ids = [f for f in store_from]
+    mutter('copy_all ids: %r', ids)
     store_to.copy_multi(store_from, ids)
 
-def hash_prefix(file_id):
-    return "%02x/" % (adler32(file_id) & 0xff)
+def hash_prefix(fileid):
+    return "%02x/" % (adler32(fileid) & 0xff)
 

=== removed file 'bzrlib/store/compressed_text.py'
--- bzrlib/store/compressed_text.py	2005-10-31 20:52:50 +0000
+++ bzrlib/store/compressed_text.py	1970-01-01 00:00:00 +0000
@@ -1,111 +0,0 @@
-# Copyright (C) 2005 by Canonical Development Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""
-An implementation the primary storage type CompressedTextStore.
-
-This store keeps compressed versions of the full text. It does not
-do any sort of delta compression.
-"""
-
-import gzip
-
-import bzrlib.store
-from bzrlib.trace import mutter
-from bzrlib.errors import BzrError, FileExists
-
-from StringIO import StringIO
-
-class CompressedTextStore(bzrlib.store.TransportStore):
-    """Store that holds files indexed by unique names.
-
-    Files can be added, but not modified once they are in.  Typically
-    the hash is used as the name, or something else known to be unique,
-    such as a UUID.
-
-    Files are stored gzip compressed, with no delta compression.
-
-    >>> st = ScratchCompressedTextStore()
-
-    >>> st.add(StringIO('hello'), 'aa')
-    >>> 'aa' in st
-    True
-    >>> 'foo' in st
-    False
-
-    You are not allowed to add an id that is already present.
-
-    Entries can be retrieved as files, which may then be read.
-
-    >>> st.add(StringIO('goodbye'), '123123')
-    >>> st.get('123123').read()
-    'goodbye'
-    """
-
-    def _relpath(self, fileid, suffixes=[]):
-        suffixes = suffixes + ['gz']
-        return super(CompressedTextStore, self)._relpath(fileid, suffixes)
-
-    def _add(self, fn, f):
-        from cStringIO import StringIO
-        from bzrlib.osutils import pumpfile
-        
-        if isinstance(f, basestring):
-            f = StringIO(f)
-            
-        sio = StringIO()
-        gf = gzip.GzipFile(mode='wb', fileobj=sio)
-        # if pumpfile handles files that don't fit in ram,
-        # so will this function
-        if isinstance(f, basestring):
-            gf.write(f)
-        else:
-            pumpfile(f, gf)
-        gf.close()
-        sio.seek(0)
-        self._transport.put(fn, sio)
-
-    def _copy_one(self, fileid, suffix, other, pb):
-        if not (isinstance(other, CompressedTextStore)
-            and other._prefixed == self._prefixed):
-            return super(CompressedTextStore, self)._copy_one(fileid, suffix, other, pb)
-        if suffix is None or suffix == 'gz':
-            path = self._relpath(fileid)
-        else:
-            path = self._relpath(fileid, [suffix])
-        result = other._transport.copy_to([path], self._transport, pb=pb)
-        assert result == 1      # or what???
-
-    def __init__(self, transport, prefixed=False):
-        super(CompressedTextStore, self).__init__(transport, prefixed)
-        self.register_suffix('gz')
-
-    def _get(self, filename):
-        """Returns a file reading from a particular entry."""
-        f = self._transport.get(filename)
-        # gzip.GzipFile.read() requires a tell() function
-        # but some transports return objects that cannot seek
-        # so buffer them in a StringIO instead
-        if hasattr(f, 'tell'):
-            return gzip.GzipFile(mode='rb', fileobj=f)
-        else:
-            from cStringIO import StringIO
-            sio = StringIO(f.read())
-            return gzip.GzipFile(mode='rb', fileobj=sio)
-
-
-def ScratchTextStore():
-    return TextStore(ScratchTransport())

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-10-17 00:46:10 +0000
+++ bzrlib/store/text.py	2005-11-10 02:11:13 +0000
@@ -21,11 +21,13 @@
 do any sort of delta compression.
 """
 
+import os
 import bzrlib.store
 from bzrlib.store import hash_prefix
 from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, NoSuchFile, FileExists
 
+import gzip
 from cStringIO import StringIO
 
 
@@ -39,11 +41,85 @@
     Files are stored uncompressed, with no delta compression.
     """
 
+    def _add_compressed(self, fn, f):
+        from cStringIO import StringIO
+        from bzrlib.osutils import pumpfile
+        
+        if isinstance(f, basestring):
+            f = StringIO(f)
+            
+        sio = StringIO()
+        gf = gzip.GzipFile(mode='wb', fileobj=sio)
+        # if pumpfile handles files that don't fit in ram,
+        # so will this function
+        pumpfile(f, gf)
+        gf.close()
+        sio.seek(0)
+        self._try_put(fn, sio)
+
     def _add(self, fn, f):
-        self._transport.put(fn, f)
+        if self._compressed:
+            self._add_compressed(fn, f)
+        else:
+            self._try_put(fn, f)
+
+    def _try_put(self, fn, f):
+        try:
+            self._transport.put(fn, f)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(os.path.dirname(fn))
+            except FileExists:
+                pass
+            self._transport.put(fn, f)
 
     def _get(self, fn):
-        return self._transport.get(fn)
+        if fn.endswith('.gz'):
+            return self._get_compressed(fn)
+        else:
+            return self._transport.get(fn)
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        # TODO: Once the copy_to interface is improved to allow a source
+        #       and destination targets, then we can always do the copy
+        #       as long as other is a TextStore
+        if not (isinstance(other, TextStore)
+            and other._prefixed == self._prefixed):
+            return super(TextStore, self)._copy_one(fileid, suffix, other, pb)
+
+        mutter('_copy_one: %r, %r', fileid, suffix)
+        path = other._get_name(fileid, suffix)
+        if path is None:
+            raise KeyError(fileid + '-' + str(suffix))
+
+        try:
+            result = other._transport.copy_to([path], self._transport, pb=pb)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(hash_prefix(fileid)[:-1])
+            except FileExists:
+                pass
+            result = other._transport.copy_to([path], self._transport, pb=pb)
+
+        if result != 1:
+            raise BzrError('Unable to copy file: %r' % (path,))
+
+    def _get_compressed(self, filename):
+        """Returns a file reading from a particular entry."""
+        f = self._transport.get(filename)
+        # gzip.GzipFile.read() requires a tell() function
+        # but some transports return objects that cannot seek
+        # so buffer them in a StringIO instead
+        if hasattr(f, 'tell'):
+            return gzip.GzipFile(mode='rb', fileobj=f)
+        else:
+            from cStringIO import StringIO
+            sio = StringIO(f.read())
+            return gzip.GzipFile(mode='rb', fileobj=sio)
 
 
 def ScratchTextStore():

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-10-24 15:56:15 +0000
+++ bzrlib/transport/__init__.py	2005-11-10 00:50:06 +0000
@@ -161,6 +161,13 @@
             yield self.has(relpath)
             count += 1
 
+    def has_any(self, relpaths):
+        """Return True if any of the paths exist."""
+        for relpath in relpaths:
+            if self.has(relpath):
+                return True
+        return False
+
     def iter_files_recursive(self):
         """Iter the relative paths of files in the transports sub-tree.
         
@@ -250,6 +257,8 @@
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
+        TODO: This interface needs to be updated so that the target location
+              can be different from the source location.
         """
         # The dummy implementation just does a simple get + put
         def copy_entry(path):

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-24 15:56:15 +0000
+++ bzrlib/transport/local.py	2005-11-10 01:31:41 +0000
@@ -183,7 +183,12 @@
             count = 0
             for path in relpaths:
                 self._update_pb(pb, 'copy-to', count, total)
-                shutil.copy(self.abspath(path), other.abspath(path))
+                try:
+                    shutil.copy(self.abspath(path), other.abspath(path))
+                except IOError, e:
+                    if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                        raise NoSuchFile('File or directory %r does not exist' % path, orig_error=e)
+                    raise LocalTransportError(orig_error=e)
                 count += 1
             return count
         else:

